<?php

namespace Modules\User\Providers;

use Illuminate\Auth\Notifications\VerifyEmail;
use Illuminate\Notifications\Messages\MailMessage;
use Illuminate\Support\Facades\Blade;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\ServiceProvider;
use Modules\User\Entities\User;
use Modules\User\Observers\UserObserver;

class UserServiceProvider extends ServiceProvider
{
    /**
     * Indicates if loading of the provider is deferred.
     *
     * @var bool
     */
    protected $defer = false;

    /**
     * Boot the application events.
     *
     * @return void
     */
    public function boot()
    {
        $this->registerTranslations();
        $this->registerConfig();
        $this->registerViews();
        $this->loadMigrationsFrom(__DIR__ . '/../Database/Migrations');

        Blade::if(
            'permission',
            function ($accessToType) {
                if (auth()->user()->isAdmin()) {
                    return true;
                }

                return auth()->user()->roles->groupPermission()->ofAccess(strtolower($accessToType[0]), $accessToType[1])->exists(
                );
            }
        );

        Validator::extendImplicit(
            'current_password',
            function ($attribute, $value, $parameters, $validator) {
                return Hash::check($value, auth()->user()->password);
            },
            trans('auth.old_password_did_not_match')
        );

        VerifyEmail::toMailUsing(
            function ($notifiable, $verificationUrl) {
                return (new MailMessage)
                    ->greeting(trans('auth.greeting', ['name' => $notifiable->name]))
                    ->subject(trans('auth.verify_email'))
                    ->line(trans('auth.click_button_verify_email'))
                    ->action(trans('auth.verify_email'), $verificationUrl)
                    ->line(trans('auth.not_create_account_no_further_action'));
            }
        );

        User::observe(UserObserver::class);
    }

    /**
     * Register the service provider.
     *
     * @return void
     */
    public function register()
    {
        $this->app->register(RouteServiceProvider::class);
    }

    /**
     * Register config.
     *
     * @return void
     */
    protected function registerConfig()
    {
        $this->publishes(
            [
                __DIR__ . '/../Config/config.php' => config_path('user.php'),
            ],
            'config'
        );
        $this->mergeConfigFrom(
            __DIR__ . '/../Config/config.php',
            'user'
        );
    }

    /**
     * Register views.
     *
     * @return void
     */
    public function registerViews()
    {
        $viewPath = resource_path('views/modules/user');

        $sourcePath = __DIR__ . '/../Resources/views';

        $this->publishes(
            [
                $sourcePath => $viewPath
            ],
            'views'
        );

        $this->loadViewsFrom(
            array_merge(
                array_map(
                    function ($path) {
                        return $path . '/modules/user';
                    },
                    \Config::get('view.paths')
                ),
                [$sourcePath]
            ),
            'user'
        );
    }

    /**
     * Register translations.
     *
     * @return void
     */
    public function registerTranslations()
    {
        $langPath = resource_path('lang/modules/user');

        if (is_dir($langPath)) {
            $this->loadTranslationsFrom($langPath, 'user');
        } else {
            $this->loadTranslationsFrom(__DIR__ . '/../Resources/lang', 'user');
        }
    }

    /**
     * Get the services provided by the provider.
     *
     * @return array
     */
    public function provides()
    {
        return [];
    }
}
